/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
 
package org.ow2.easywsdl.extensions.multiple;

import java.util.List;
import java.util.Map;

import org.ow2.easywsdl.extensions.multiple.api.Description;
import org.ow2.easywsdl.extensions.multiple.api.MultipleExtException;
import org.ow2.easywsdl.extensions.multiple.api.MultipleExtReader;
import org.ow2.easywsdl.extensions.multiple.api.MultipleExtWriter;
import org.ow2.easywsdl.wsdl.ExtensionFactory;
import org.ow2.easywsdl.wsdl.api.WSDLReader.FeatureConstants;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription;


/**
 * This abstract class defines a factory API that enables applications to obtain
 * a WSDLFactory capable of producing new Definitions, new WSDLReaders, and new
 * WSDLWriters.
 * 
 * <p>
 * Some ideas used here have been shamelessly copied from the wonderful JAXP and
 * Xerces work.
 * </p>
 *
 * @author Nicolas Boissel-Dallier - EBM WebSourcing
 */
public abstract class MultipleExtFactory {

    /**
     * Get a new instance of a WSDLFactory. This method follows (almost) the
     * same basic sequence of steps that JAXP follows to determine the
     * fully-qualified class name of the class which implements WSDLFactory.
     * <p>
     * The steps in order are:
     * <ol>
     * <li>Check the property file
     * META-INF/services/javax.wsdl.factory.WSDLFactory.</li>
     * <li>Check the javax.wsdl.factory.WSDLFactory system property.</li>
     * <li>Check the lib/wsdl.properties file in the JRE directory. The key will
     * have the same name as the above system property.</li>
     * <li>Use the default class name provided by the implementation.</li>
     * </ol>
     * <p>
     * Once an instance of a MultipleExtFactory is obtained, newMultipleExtReader(), 
     * or newMultipleExtWriter(), to create the desired instances.
     */
    public static MultipleExtFactory newInstance() throws MultipleExtException {
        return new MultipleExtFactoryImpl();
    }

    /**
     * Create a new instance of a Definition.
     * 
     * @throws MultipleExtException
     */
    public abstract Description addMultipleExtElmt2Description(List<ExtensionFactory> absFactories, AbsItfDescription desc)
            throws MultipleExtException;

    /**
     * Create a new instance of a MultipleExtReaderImpl.
     * 
     * @throws MultipleExtException
     */
    public abstract MultipleExtReader newMultipleExtReader() throws MultipleExtException;

    /**
     * Create a new instance of a MultipleExtReaderImpl.
     * 
     * @throws MultipleExtException
     */
    public abstract MultipleExtReader newMultipleExtReader(Map<FeatureConstants, Object> features)
            throws MultipleExtException;

    /**
     * Create a new instance of a MultipleExtWriterImpl.
     * 
     * @throws MultipleExtException
     */
    public abstract MultipleExtWriter newMultipleExtWriter() throws MultipleExtException;
}
